var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
var __rest = (this && this.__rest) || function (s, e) {
    var t = {};
    for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p) && e.indexOf(p) < 0)
        t[p] = s[p];
    if (s != null && typeof Object.getOwnPropertySymbols === "function")
        for (var i = 0, p = Object.getOwnPropertySymbols(s); i < p.length; i++) {
            if (e.indexOf(p[i]) < 0 && Object.prototype.propertyIsEnumerable.call(s, p[i]))
                t[p[i]] = s[p[i]];
        }
    return t;
};
import React, { useState, useEffect, useRef } from 'react';
import classNames from 'classnames';
import Input from '../Input/input';
import Icon from '../Icon/icon';
import useDebounce from '../../hooks/useDebounce';
import useClickOutside from '../../hooks/useClickOutside';
import Transition from '../Transition/transition';
/**
 * ## 引用方法
 * ~~~js
 * import {AutoComplete} from 'wsq'
 * ~~~
 */
export var AutoComplete = function (props) {
    var fetchSuggestions = props.fetchSuggestions, onSelect = props.onSelect, value = props.value, renderOption = props.renderOption, restProps = __rest(props
    // 用户输入的值
    , ["fetchSuggestions", "onSelect", "value", "renderOption"]);
    // 用户输入的值
    var _a = useState(value), inputVal = _a[0], setInputVal = _a[1];
    // 关联的数据
    var _b = useState([]), suggestions = _b[0], setSuggestions = _b[1];
    //
    var _c = useState(false), showDropdown = _c[0], setShowDropdown = _c[1];
    // 加载动画
    var _d = useState(false), loading = _d[0], setLoading = _d[1];
    // 高亮显示选中的项
    var _e = useState(-1), highlightIndex = _e[0], setHighlightIndex = _e[1];
    // 防止选中某一项之后 又触发一次请求关联数据
    // 上述bug出现的原因是：我们监听用户输入的值的变化去请求关联数据 这个动作在两个地方有执行 一个是handleChange 一个是handleSelect
    // 在handleChange的时候 设置triggerSearch为true 表示可以搜索关联数据
    // 在handleSelect的时候 设置triggerSearch为false 表示禁止搜索关联数据
    var triggerSearch = useRef(false);
    // 自定义hook useDebounce函数防抖
    var debouncedValue = useDebounce(inputVal, 500);
    // 点击空白处 让关联列表消失
    var componentRef = useRef(null);
    // 调用自定义hook useClickOutside
    useClickOutside(componentRef, function () {
        setSuggestions([]);
        setShowDropdown(false);
    });
    // 监听经过处理用户输入的值和延时时间得出来的结果 debouncedValue 
    useEffect(function () {
        if (debouncedValue && triggerSearch.current) {
            var results = fetchSuggestions(debouncedValue);
            // console.log(results)
            // 异步请求数据
            if (results instanceof Promise) {
                // 加载动画开启
                setLoading(true);
                results.then(function (data) {
                    // 加载动画关闭
                    setLoading(false);
                    // 更新关联数据
                    setSuggestions(data);
                    if (data.length > 0) {
                        setShowDropdown(true);
                    }
                });
            }
            else {
                // 更新关联数据
                setSuggestions(results);
                if (results.length > 0) {
                    setShowDropdown(true);
                }
            }
        }
        else {
            setShowDropdown(false);
            // setSuggestions([])
        }
        setHighlightIndex(-1);
        // 依赖inputVal这个值
    }, [debouncedValue, fetchSuggestions]);
    // 将当前index传进来
    var highlight = function (index) {
        if (index < 0)
            index = 0;
        if (index >= suggestions.length) {
            index = suggestions.length - 1;
        }
        setHighlightIndex(index);
    };
    // 监听键盘
    var hangleKeyDown = function (e) {
        // console.log(e)
        switch (e.keyCode) {
            case 13:
                //enter
                if (suggestions[highlightIndex]) {
                    handleSelect(suggestions[highlightIndex]);
                }
                break;
            case 38:
                // 向上
                highlight(highlightIndex - 1);
                break;
            case 40:
                // 向下
                highlight(highlightIndex + 1);
                break;
            case 27:
                // esc
                // setSuggestions([])
                setShowDropdown(false);
                break;
            default:
                break;
        }
    };
    // 根据输入的值匹配关联数据列表
    var handleChange = function (e) {
        // 去空格
        var value = e.target.value.trim();
        setInputVal(value);
        triggerSearch.current = true;
        // 若value存在 则更新关联列表的数据
        // if (value) {
        //   const results = fetchSuggestions(value)
        //   // console.log(results)
        //   // results 此时是个联合类型
        //   if (results instanceof Promise) {
        //     setLoading(true)
        //     results.then(data => {
        //       setLoading(false)
        //       setSuggestions(data)
        //     })
        //   } else {
        //     setSuggestions(results)
        //   }
        // } else {
        //   setSuggestions([])
        // }
    };
    // 选择关联数据其中一项
    var handleSelect = function (item) {
        setInputVal(item.value);
        // setSuggestions([])
        setShowDropdown(false);
        if (onSelect) {
            onSelect(item);
        }
        triggerSearch.current = false;
    };
    // 用户自定义模版内容
    var renderTemplate = function (item) {
        return renderOption ? renderOption(item) : item.value;
    };
    // 返回关联数据列表html
    var generateDropdown = function () {
        return (React.createElement(Transition, { in: showDropdown || loading, animation: "zoom-in-top", timeout: 300, onExited: function () { console.log(1111111); setSuggestions([]); } },
            React.createElement("ul", { className: "wsq-suggestion-list" },
                loading &&
                    React.createElement("div", { className: "suggstions-loading-icon" },
                        React.createElement(Icon, { icon: "spinner", spin: true })),
                suggestions.map(function (item, index) {
                    var cnames = classNames('suggestion-item', {
                        'item-highlighted': index === highlightIndex
                    });
                    return (React.createElement("li", { key: index, className: cnames, onClick: function () { return handleSelect(item); } }, renderTemplate(item)));
                }))));
    };
    return (React.createElement("div", { className: "wsq-auto-complete", ref: componentRef },
        React.createElement(Input, __assign({ value: inputVal, onChange: handleChange, onKeyDown: hangleKeyDown }, restProps)),
        suggestions && generateDropdown()));
};
AutoComplete.defaultProps = {
    disabled: false
};
export default AutoComplete;
