import React, { useState, createContext } from 'react';
import classNames from 'classnames';
// 创建上下文
export var MenuContext = createContext({ index: '0' });
var Menu = function (props) {
    var 
    // 取出className
    className = props.className, defaultIndex = props.defaultIndex, mode = props.mode, // horizontal 横向菜单 ｜ vertical 纵向菜单
    style = props.style, children = props.children, onSelect = props.onSelect, defaultOpenSubMenus = props.defaultOpenSubMenus;
    var _a = useState(defaultIndex), currentActive = _a[0], setActive = _a[1];
    // 用户有自定义classname 就把它放进去
    var classes = classNames('wsq-menu', className, {
        'menu-vertical': mode === 'vertical',
        'menu-horizontal': mode !== 'vertical'
    });
    // 传给子组件使用的点击事件，index是子组件传过来的值
    var handleClick = function (index) {
        // 重置当前的index
        setActive(index);
        // 如果menu组件有传onSelect属性 那就执行
        if (onSelect) {
            onSelect(index);
        }
    };
    // 定义传给子组件的数据
    var passedContext = {
        index: currentActive ? currentActive : '0',
        onSelect: handleClick,
        mode: mode,
        defaultOpenSubMenus: defaultOpenSubMenus
    };
    var renderChildren = function () {
        // 将子节点循环
        return React.Children.map(children, function (child, index) {
            // 将节点断言成MenuItemProps类型
            var childElement = child;
            // 获取子节点定义的displayname
            var displayName = childElement.type.displayName;
            // 如果节点的 displayName === MenuItem 则返回 child
            if (displayName === 'MenuItem' || displayName === 'SubMenu') {
                // 复制childElement里的index 并返回
                // console.log(index)
                return React.cloneElement(childElement, {
                    index: index.toString()
                });
            }
            else {
                console.error('warning: menu has a child');
            }
        });
    };
    return (React.createElement("ul", { className: classes, style: style, "data-testid": "test-menu" },
        React.createElement(MenuContext.Provider, { value: passedContext }, renderChildren())));
};
Menu.defaultProps = {
    defaultIndex: '0',
    mode: 'horizontal',
    defaultOpenSubMenus: []
};
export default Menu;
