var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
var __spreadArray = (this && this.__spreadArray) || function (to, from, pack) {
    if (pack || arguments.length === 2) for (var i = 0, l = from.length, ar; i < l; i++) {
        if (ar || !(i in from)) {
            if (!ar) ar = Array.prototype.slice.call(from, 0, i);
            ar[i] = from[i];
        }
    }
    return to.concat(ar || Array.prototype.slice.call(from));
};
import React, { useRef, useState } from 'react';
import axios from 'axios';
import Dragger from './dragger';
import UploadList from './uploadList';
export var Upload = function (props) {
    var action = props.action, defaultFileList = props.defaultFileList, onRemove = props.onRemove, beforeUpload = props.beforeUpload, onChange = props.onChange, onProgress = props.onProgress, onSuccess = props.onSuccess, onError = props.onError, headers = props.headers, name = props.name, data = props.data, withCredentials = props.withCredentials, accept = props.accept, multiple = props.multiple, drag = props.drag, children = props.children;
    // 绑定dom节点
    var fileInupt = useRef(null);
    // 存放上传的文件信息
    var _a = useState(defaultFileList || []), fileList = _a[0], setFileList = _a[1];
    // 更新上传的文件信息里的内容 比如进度
    // updateFile：要更新的项 updateObj：要更新的值  Partial代表要更新updateFile里的任意几项都可以 可选
    var updateFileList = function (updateFile, updateObj) {
        setFileList(function (prevList) {
            return prevList.map(function (file) {
                if (file.uid === updateFile.uid) {
                    return __assign(__assign({}, file), updateObj);
                }
                else {
                    return file;
                }
            });
        });
    };
    var handleClick = function () {
        if (fileInupt.current) {
            // 调用input的点击事件
            fileInupt.current.click();
        }
    };
    // 选择文件后进行的操作
    var handleFileChange = function (e) {
        var files = e.target.files;
        if (!files) {
            return;
        }
        uploadFiles(files);
        // 为何要清空？
        if (fileInupt.current) {
            fileInupt.current.value = '';
        }
    };
    // 删除
    var handleRemove = function (file) {
        console.log(file);
        setFileList(function (prevList) {
            return prevList.filter(function (item) { return item.uid !== file.uid; });
        });
        if (onRemove) {
            onRemove(file);
        }
    };
    // 上传文件
    var uploadFiles = function (files) {
        // 将FileList类型的files 转成数组
        var postFiles = Array.from(files);
        postFiles.forEach(function (file) {
            if (!beforeUpload) { // 如果beforeUpload没传 那就直接上传文件
                post(file);
            }
            else {
                var result = beforeUpload(file);
                if (result && result instanceof Promise) { // 如果result存在 并且是个Promise
                    result.then(function (processedFile) {
                        post(processedFile);
                    });
                }
                else if (result === true) { // 如果是个boolean值 且是true
                    post(file);
                }
            }
        });
    };
    var post = function (file) {
        var _file = {
            uid: Date.now() + 'upload-file',
            status: 'ready',
            name: file.name,
            size: file.size,
            percent: 0,
            raw: file
        };
        // 使用这种方式会出现bug，上传多个文件 列表里只显示1个 所以要改用回调的方式
        // setFileList([_file, ...fileList])
        setFileList(function (prevList) {
            return __spreadArray([_file], prevList, true);
        });
        var formData = new FormData();
        // 如果有传name 则用到文件名上
        formData.append(name || file.name, file);
        // 添加用户要传的参数
        if (data) {
            Object.keys(data).forEach(function (key) {
                formData.append(key, data[key]);
            });
        }
        axios.post(action, formData, {
            headers: __assign(__assign({}, headers), { 'Content-ype': 'multipart/form-data' }),
            // withCredentials
            withCredentials: withCredentials,
            onUploadProgress: function (e) {
                // console.log(e)
                // percentage表示进度 数字类型
                var percentage = Math.round((e.loaded * 100) / e.total) || 0;
                // console.log(percentage)
                // useState 可以传入函数
                setFileList(function (preList) {
                    console.log(preList);
                    return preList;
                });
                if (percentage < 100) {
                    // 更新上传文件列表里文件的进度状态
                    updateFileList(_file, { percent: percentage, status: 'uploading' });
                    if (onProgress) {
                        onProgress(percentage, file);
                    }
                }
            }
        }).then(function (resp) {
            // console.log(resp)
            // console.log(file)
            updateFileList(_file, { status: 'success', response: resp.data });
            if (onSuccess) {
                onSuccess(resp.data, file);
            }
            if (onChange) {
                onChange(file);
            }
        }).catch(function (err) {
            updateFileList(_file, { status: 'error', response: err });
            console.error(err);
            if (onError) {
                onError(err, file);
            }
            if (onChange) {
                onChange(file);
            }
        });
    };
    console.log(fileList);
    return (React.createElement("div", { className: "wsq-upload-component" },
        React.createElement("div", { className: "wsq-upload-input", style: { display: 'inline-block' }, onClick: handleClick },
            drag ?
                React.createElement(Dragger, { onFile: function (files) { uploadFiles(files); } }, children) : children,
            React.createElement("input", { className: "wsq-file-input", style: { display: 'none' }, ref: fileInupt, onChange: handleFileChange, type: "file", accept: accept, multiple: multiple })),
        React.createElement(UploadList, { fileList: fileList, onRemove: handleRemove })));
};
Upload.defaultProps = {
    name: 'file'
};
export default Upload;
