import { useState, useEffect } from 'react';
function useDebounce(value, delay) {
    if (delay === void 0) { delay = 300; }
    var _a = useState(value), debouncedValue = _a[0], setDebouncedValue = _a[1];
    // 根据用户输入的value和delay的变化执行useEffect里的内容
    // 如果用户输入文字的间隔大于等于delay的值 则创建一个延时 debouncedValue的值被更新
    // 如果用户输入文字的间隔时间小于delay的值 则清除effect的函数，延时被clearTimeout debouncedValue未被更新 接着又创建一个延时 debouncedValue的值被更新
    useEffect(function () {
        // 一段时间后更新 debouncedValue 的值
        var handler = window.setTimeout(function () {
            setDebouncedValue(value);
        }, delay);
        // 当useEffect里添加回调函数return func 的时候 就是表示清除副作用
        return function () {
            clearTimeout(handler);
        };
        // 在value和delay改变的时候执行 优先级按先后
    }, [value, delay]);
    return debouncedValue;
}
export default useDebounce;
